# -*- coding: utf-8 -*-

import abc
import argparse
import datetime
import itertools
import json
import os

from safari import Safari


class Exporter(object):
    @abc.abstractmethod
    def export(self, filename):
        raise NotImplementedError()


class OrgExporter(Exporter):
    def format_org_link(self, title, url):
        timestamp = datetime.datetime.now().strftime("%Y-%m-%d %a %H:%M")

        return f"* [[{url}][{title}]]\n[{timestamp}]"

    def export(self, results, filename):
        with open(filename, mode="w") as f:
            f.writelines(
                f"{self.format_org_link(**item)}\n\n"
                for item in itertools.chain.from_iterable(results.values())
            )


class JsonExporter(Exporter):
    def export(self, results, filename):
        with open(filename, mode="w") as f:
            json.dump(results, f, ensure_ascii=False, indent=4)


def parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "-s",
        "--source",
        type=str,
        default="bookmarks",
        help="Bookmark types ('bookmarks', 'readings', 'all') (default: 'bookmarks')",
    )
    parser.add_argument(
        "-t",
        "--target",
        type=str,
        default="safari_exported.json",
        help="Output file name. (default 'safari_exported.json')",
    )
    parser.add_argument(
        "-l",
        "--library",
        type=str,
        default="~/Library/Safari/",
        help="Safari library location (default: '~/Library/Safari/')",
    )
    args = parser.parse_args()

    return args


def main():
    args = parse_args()

    # Fetch bookmarks.
    safari = Safari(os.path.abspath(os.path.expanduser(args.library)))
    sources = {"bookmarks", "readings"}
    if args.source != "all":
        sources = {args.source}
    results = {source: list(getattr(safari, source)) for source in sources}

    # Export bookmarks.
    exporters = {
        ".org": OrgExporter(),
        ".json": JsonExporter(),
    }
    ext = os.path.splitext(args.target)[1]
    if not ext:
        ext = ".org"
    exporter = exporters.get(ext)
    if not exporter:
        raise TypeError(f"Unsupported file type: {ext}")
    exporter.export(results, args.target)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-

import os
import plistlib
import sqlite3


class Safari(object):
    def __init__(self, library=os.path.join(os.environ["HOME"], "Library/Safari/")):
        self.bookmark_file = os.path.join(library, "Bookmarks.plist")
        self.db_file = os.path.join(library, "CloudTabs.db")

    @property
    def readings(self):
        with open(self.bookmark_file, mode="rb") as plist_file:
            plist = plistlib.load(plist_file)

        bookmarks = None
        for child in plist["Children"]:
            if child.get("Title", None) == "com.apple.ReadingList":
                bookmarks = child
                break

        if not bookmarks:
            return

        for bookmark in bookmarks["Children"]:
            yield {
                "title": bookmark["URIDictionary"]["title"],
                "url": bookmark["URLString"],
            }

    @property
    def bookmarks(self):
        with sqlite3.connect(self.db_file) as conn:
            c = conn.cursor()

            sql_iphone_device = (
                'select device_uuid from cloud_tab_devices where device_name="iPhone";'
            )
            iphone_id = next((x[0] for x in c.execute(sql_iphone_device)), None,)
            if not iphone_id:
                raise RuntimeError("iPhone device_id not found")

            sql_links = (
                f'select title, url from cloud_tabs where device_uuid="{iphone_id}";'
            )
            tabs = c.execute(sql_links)

            for tab in tabs:
                yield {
                    "title": tab[0],
                    "url": tab[1],
                }
